package edu.travel.tenant.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import edu.travel.remote.feign.mode.dto.tenant.AddMenuDto;
import edu.travel.remote.feign.mode.vo.tenant.AddMenuVo;
import edu.travel.remote.feign.mode.vo.tenant.MenuTreeVo;
import edu.travel.remote.feign.mode.vo.tenant.SysMenuExtraDataVo;
import edu.travel.remote.feign.mode.vo.tenant.SysMenuVo;
import edu.travel.service.SysServiceImpl;
import edu.travel.tenant.entity.*;
import edu.travel.tenant.mapper.SysMenuMapper;
import edu.travel.tenant.service.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.stream.Collectors;

/**
 * ISysRoleServiceImpl 类。
 * <p>
 * 描述：
 *
 * @author huangwenwen
 * @date 2025/2/6
 */

@Service
public class ISysMenuServiceImpl extends SysServiceImpl<SysMenuMapper, SysMenu> implements ISysMenuService {

    @Autowired
    private ISysMenuExtraDataService sysMenuExtraDataService;
    @Autowired
    private ISysRoleMenuService sysRoleMenuService;
    @Autowired
    private ISysRoleService sysRoleService;

    @Override
    @Transactional
    public AddMenuVo addMenu(AddMenuDto addMenuDto) {
        //参数校验
        Assert.isTrue(ObjectUtil.isNotEmpty(addMenuDto),"参数不能为空");
        Assert.isTrue(ObjectUtil.isNotEmpty(addMenuDto.getSysMenu()),"参数不能为空");
        Assert.isTrue(ObjectUtil.isNotEmpty(addMenuDto.getSysMenu().getPath()),"页面路由不能为空");
        Assert.isTrue(ObjectUtil.isNotEmpty(addMenuDto.getSysMenu().getComponent()),"组件路径不能为空");
        //保存菜单
        super.save(BeanUtil.copyProperties(addMenuDto.getSysMenu(),SysMenu.class));
        addMenuDto.getSysMenuExtraData().setMenuId(addMenuDto.getSysMenu().getId());
        sysMenuExtraDataService.save(BeanUtil.copyProperties(addMenuDto.getSysMenuExtraData(), SysMenuExtraData.class));

        //将菜单分配给超级管理员
        EduTenant principal =(EduTenant) SecurityContextHolder.getContext().getAuthentication().getPrincipal();
        SysRoleMenu sysRoleMenu = new SysRoleMenu();
        sysRoleMenu.setRoleId(sysRoleService.getOne(new LambdaQueryWrapper<SysRole>().eq(SysRole::getName,"超级管理员")).getId());
        sysRoleMenu.setMenuId(addMenuDto.getSysMenuExtraData().getMenuId());
        sysRoleMenu.setProject(principal.getProject());
        sysRoleMenuService.save(sysRoleMenu);

        AddMenuVo addMenuVo = new AddMenuVo();
        addMenuVo.setSysMenu(BeanUtil.copyProperties(addMenuDto.getSysMenu(), SysMenuVo.class));
        addMenuVo.setSysMenuExtraData(BeanUtil.copyProperties(addMenuDto.getSysMenuExtraData(), SysMenuExtraDataVo.class));
        return addMenuVo;
    }

    @Override
    @Transactional
    public void deleteMenuByIds(Set<Long> ids) {
        //参数校验
        Assert.isTrue(ObjectUtil.isNotEmpty(ids),"请选择想要删除的菜单");
        List<SysMenu> sysMenus = super.listByIds(ids);
        Assert.isTrue(ObjectUtil.isNotEmpty(sysMenus),"选择的菜单不存在或已删除");
        Assert.isTrue(sysMenus.size() == ids.size(),"包含重复的菜单或已删除菜单");
        sysMenus.forEach(sysMenu -> {
            sysMenu.setDeleteFlag(1);
        });
        SysMenuExtraData sysMenuExtraData = new SysMenuExtraData();
        sysMenuExtraData.setDeleteFlag(1);
        sysMenuExtraDataService.remove(new LambdaQueryWrapper<SysMenuExtraData>().in(SysMenuExtraData::getMenuId, ids));
        this.removeByIds(ids);
    }

    @Override
    public void updateMenuById(AddMenuDto updateMenuDto) {
        Assert.isTrue(ObjectUtil.isNotEmpty(updateMenuDto),"参数不能为空");
        Assert.isTrue(ObjectUtil.isNotEmpty(updateMenuDto.getSysMenu().getId()),"id不能为空");
        Assert.isTrue(ObjectUtil.isNotEmpty(updateMenuDto.getSysMenuExtraData().getId()),"id不能为空");
        Assert.isTrue(ObjectUtil.isNotEmpty(updateMenuDto.getSysMenu().getPath()),"页面路由不能为空");
        Assert.isTrue(ObjectUtil.isNotEmpty(updateMenuDto.getSysMenu().getComponent()),"组件路径不能为空");
        SysMenuExtraData extraData = sysMenuExtraDataService.getById(updateMenuDto.getSysMenuExtraData().getId());
        Assert.isTrue(Objects.equals(extraData.getId(), updateMenuDto.getSysMenuExtraData().getId()),"该附加信息与菜单不匹配");
        //保存菜单
        super.updateById(BeanUtil.copyProperties(updateMenuDto.getSysMenu(),SysMenu.class));
        sysMenuExtraDataService.updateById(BeanUtil.copyProperties(updateMenuDto.getSysMenuExtraData(), SysMenuExtraData.class));
    }

    @Override
    public List<MenuTreeVo> getMenuTreeListAll(List<SysMenu> records) {
        // 获取菜单ID集合
        List<Long> menuIdList = records.stream()
                .map(SysMenu::getId)
                .collect(Collectors.toList());

        // 获取菜单附加信息
        List<SysMenuExtraData> menuExtraDataList = sysMenuExtraDataService.list(new LambdaQueryWrapper<SysMenuExtraData>()
                .in(SysMenuExtraData::getMenuId, menuIdList));

        // 构建菜单树 菜单list 附加list
        List<MenuTreeVo> treeList = buildMenuTree(records, BeanUtil.copyToList(menuExtraDataList, SysMenuExtraDataVo.class));

        return treeList;
    }

    /**
     * 构建菜单树
     */
    @Override
    public List<MenuTreeVo> buildMenuTree(List<SysMenu> sysMenuList, List<SysMenuExtraDataVo> menuExtraDataList) {
        Map<Long, MenuTreeVo> menuMap = new HashMap<>();
        List<MenuTreeVo> rootMenus = new ArrayList<>();

        // 构建每个菜单的 MenuTreeVo
        sysMenuList.forEach(sysMenu -> {
            MenuTreeVo menuTreeVo = BeanUtil.copyProperties(sysMenu, MenuTreeVo.class);
            menuMap.put(sysMenu.getId(), menuTreeVo);
        });

        // 给每个菜单节点加子菜单
        sysMenuList.forEach(sysMenu -> {
            if (sysMenu.getParentId() == null) {
                rootMenus.add(menuMap.get(sysMenu.getId()));  // 根节点
            } else {
                MenuTreeVo parentMenu = menuMap.get(sysMenu.getParentId());
                if (parentMenu != null) {
                    parentMenu.getChildren().add(menuMap.get(sysMenu.getId()));
                }
            }
        });

        // 给菜单附加额外数据
        menuExtraDataList.forEach(extraData -> {
            MenuTreeVo menuTreeVo = menuMap.get(extraData.getMenuId());
            if (menuTreeVo != null) {
                menuTreeVo.setExtraData(extraData);
            }
        });

        return rootMenus;
    }

}
